﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using HeuristicLab.Clients.Common;
using HeuristicLab.Collections;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.Clients.OKB {
  [Item("Administrator", "OKB administrator front-end.")]
  public sealed class Administrator : IContent {
    private static Administrator instance;
    public static Administrator Instance {
      get {
        if (instance == null) instance = new Administrator();
        return instance;
      }
    }

    private ItemCollection<Platform> platforms;
    public ItemCollection<Platform> Platforms {
      get { return platforms; }
    }
    private ItemCollection<AlgorithmClass> algorithmClasses;
    public ItemCollection<AlgorithmClass> AlgorithmClasses {
      get { return algorithmClasses; }
    }
    private ItemCollection<Algorithm> algorithms;
    public ItemCollection<Algorithm> Algorithms {
      get { return algorithms; }
    }

    private Administrator() { }

    public void Refresh() {
      OnRefreshing();
      if (platforms == null) {
        platforms = new ItemCollection<Platform>();
        platforms.ItemsRemoved += new CollectionItemsChangedEventHandler<Platform>(platforms_ItemsRemoved);
      }
      platforms.Clear();
      if (algorithmClasses == null) {
        algorithmClasses = new ItemCollection<AlgorithmClass>();
        algorithmClasses.ItemsRemoved += new CollectionItemsChangedEventHandler<AlgorithmClass>(algorithmClasses_ItemsRemoved);
      }
      algorithmClasses.Clear();
      if (algorithms == null) {
        algorithms = new ItemCollection<Algorithm>();
        algorithms.ItemsRemoved += new CollectionItemsChangedEventHandler<Algorithm>(algorithms_ItemsRemoved);
      }
      algorithms.Clear();

      var call = new Func<Exception>(delegate() {
        try {
          platforms.AddRange(CallAdminService<ItemCollection<Platform>>(s => s.GetPlatforms()).OrderBy(a => a.Name));
          algorithmClasses.AddRange(CallAdminService<ItemCollection<AlgorithmClass>>(s => s.GetAlgorithmClasses()).OrderBy(a => a.Name));
          algorithms.AddRange(CallAdminService<ItemCollection<Algorithm>>(s => s.GetAlgorithms()).OrderBy(a => a.Name));
          return null;
        }
        catch (Exception ex) {
          return ex;
        }
      });
      call.BeginInvoke(delegate(IAsyncResult result) {
        Exception ex = call.EndInvoke(result);
        if (ex != null) ErrorHandling.ShowErrorDialog("Refresh failed.", ex);
        OnRefreshed();
      }, null);
    }

    public bool Store(IOKBItem item) {
      try {
        if (item is Platform)
          CallAdminService(s => s.StorePlatform((Platform)item));
        else if (item is AlgorithmClass)
          CallAdminService(s => s.StoreAlgorithmClass((AlgorithmClass)item));
        else if (item is Algorithm)
          CallAdminService(s => s.StoreAlgorithm((Algorithm)item));
        return true;
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Store failed.", ex);
        return false;
      }
    }

    public event EventHandler Refreshing;
    private void OnRefreshing() {
      EventHandler handler = Refreshing;
      if (handler != null) handler(this, EventArgs.Empty);
    }
    public event EventHandler Refreshed;
    private void OnRefreshed() {
      EventHandler handler = Refreshed;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    private void platforms_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<Platform> e) {
      try {
        foreach (Platform p in e.Items)
          CallAdminService(s => s.DeletePlatform(p.Id));
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Delete failed.", ex);
      }
    }
    private void algorithmClasses_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<AlgorithmClass> e) {
      try {
        foreach (AlgorithmClass a in e.Items)
          CallAdminService(s => s.DeleteAlgorithmClass(a.Id));
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Delete failed.", ex);
      }
    }
    private void algorithms_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<Algorithm> e) {
      try {
        foreach (Algorithm a in e.Items)
          CallAdminService(s => s.DeleteAlgorithm(a.Id));
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog("Delete failed.", ex);
      }
    }

    #region Helpers
    private void CallAdminService(Action<IAdminService> call) {
      AdminServiceClient client = ClientFactory.CreateClient<AdminServiceClient, IAdminService>();
      try {
        call(client);
      }
      finally {
        try {
          client.Close();
        }
        catch (Exception) {
          client.Abort();
        }
      }
    }
    private T CallAdminService<T>(Func<IAdminService, T> call) where T : class {
      AdminServiceClient client = ClientFactory.CreateClient<AdminServiceClient, IAdminService>();
      try {
        return call(client);
      }
      finally {
        try {
          client.Close();
        }
        catch (Exception) {
          client.Abort();
        }
      }
    }
    #endregion
  }
}
